@@ -44,7 +44,6 @@
 #include <libgnome/libgnome.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <eel/eel-vfs-extensions.h>
-#include <eel/eel-glib-extensions.h>
 
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 
@@ -186,8 +185,6 @@ gedit_document_release_untitled_number (gint n)
 	g_return_if_fail (ret);	
 }
 
-
-
 GType
 gedit_document_get_type (void)
 {
@@ -1033,7 +1030,8 @@ get_info_cb (GnomeVFSAsyncHandle *handle,
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
-	g_return_if_fail (eel_g_list_exactly_one_item (results));
+	/* assert that the list has one and only one item */
+	g_return_if_fail (results != NULL && results->next == NULL);
 
 	doc = GEDIT_DOCUMENT (callback_data);
 	doc->priv->info_handle = NULL;
@@ -1065,6 +1063,33 @@ get_info_cb (GnomeVFSAsyncHandle *handle,
 		return;
 	}
 
+	/* just some extra paranoia, since we passed the follow-symlink flag... */
+	g_return_if_fail (info_result->file_info->type != GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK);
+
+	/* if it's not a regular file, error out... */
+	if (info_result->file_info->type != GNOME_VFS_FILE_TYPE_REGULAR)
+	{
+		GError *error = NULL;
+
+		gedit_debug (DEBUG_DOCUMENT, "File %s is of type: %d",
+			     doc->priv->temp_uri, info_result->file_info->type);
+
+		reset_temp_data (doc);
+
+		error = g_error_new (GEDIT_DOCUMENT_IO_ERROR, 
+				     GEDIT_ERROR_NOT_REGULAR_FILE,
+				     "Not a regular file");
+
+		g_signal_emit (G_OBJECT (doc), 
+			       document_signals[LOADED], 
+			       0,
+			       error);
+
+		g_error_free (error);
+
+		return;
+	}
+
 	if (info_result->file_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)
 	{
 		doc->priv->temp_size = (gulong)info_result->file_info->size;
@@ -1090,22 +1115,80 @@ static gboolean
 load_local (GeditDocument *doc)
 {
 	GError *error = NULL;
+	GnomeVFSFileInfo *info;
 	GnomeVFSResult res;
 	gint file_size;
 	gchar *file_contents;
 	gboolean ret;
-	
+
 	PROFILE (
 		g_message ("Start loading entire file %s: %.3f", 
 			doc->priv->uri, g_timer_elapsed (timer, NULL));
 	)
 
+	info = gnome_vfs_file_info_new ();
+
+	res = gnome_vfs_get_file_info (doc->priv->temp_uri,
+				       info,
+				       GNOME_VFS_FILE_INFO_DEFAULT |
+				       GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+
+	if (res != GNOME_VFS_OK)
+	{
+		reset_temp_data (doc);
+
+		gnome_vfs_file_info_unref (info);
+
+		error = g_error_new (GEDIT_DOCUMENT_IO_ERROR, 
+				     res,
+				     gnome_vfs_result_to_string (res));
+
+		g_signal_emit (G_OBJECT (doc), 
+			       document_signals[LOADED], 
+			       0,
+			       error);
+
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	/* just some extra paranoia, since we passed the follow-symlink flag... */
+	g_return_val_if_fail (info->type != GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK, FALSE);
+
+	/* if it's not a regular file, error out... */
+	if (info->type != GNOME_VFS_FILE_TYPE_REGULAR)
+	{
+		GError *error = NULL;
+
+		gedit_debug (DEBUG_DOCUMENT, "File %s is of type: %d",
+			     doc->priv->temp_uri, info->type);
+
+		reset_temp_data (doc);
+
+		error = g_error_new (GEDIT_DOCUMENT_IO_ERROR, 
+				     GEDIT_ERROR_NOT_REGULAR_FILE,
+				     "Not a regular file");
+
+		g_signal_emit (G_OBJECT (doc), 
+			       document_signals[LOADED], 
+			       0,
+			       error);
+
+		g_error_free (error);
+		gnome_vfs_file_info_unref (info);
+
+		return FALSE;
+	}
+
+	gnome_vfs_file_info_unref (info);
+
 	res = gnome_vfs_read_entire_file (doc->priv->temp_uri, 
 					  &file_size, 
 					  &file_contents);
 
 	if (res != GNOME_VFS_OK)
-	{		
+	{
 		reset_temp_data (doc);
 		
 		g_free (file_contents);
@@ -1354,7 +1437,6 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	return TRUE;
 }
 
-
 gboolean 
 gedit_document_is_untouched (const GeditDocument *doc)
 {
@@ -1735,7 +1817,7 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri, const GeditEn
 		gchar *error_message;
 		gchar *scheme_string;
 
-		gchar *temp = eel_uri_get_scheme (uri);
+		gchar *temp = gnome_vfs_get_uri_scheme (uri);
                 scheme_string = eel_make_valid_utf8 (temp);
 		g_free (temp);
 
@@ -2096,7 +2178,6 @@ gedit_document_insert_text_at_cursor (GeditDocument *doc, const gchar *text, gin
 	gtk_text_buffer_insert_at_cursor (GTK_TEXT_BUFFER (doc), text, len);
 }
 
-
 void 
 gedit_document_delete_text (GeditDocument *doc, gint start, gint end)
 {
@@ -2248,7 +2329,6 @@ gedit_document_end_user_action (GeditDocument *doc)
 	gtk_text_buffer_end_user_action (GTK_TEXT_BUFFER (doc));
 }
 
-
 void
 gedit_document_goto_line (GeditDocument* doc, guint line)
 {
@@ -2342,7 +2422,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str, gint flags)
 	gedit_debug (DEBUG_DOCUMENT, "converted_str: %s", converted_str);
 
 	search_flags = GTK_SOURCE_SEARCH_VISIBLE_ONLY | GTK_SOURCE_SEARCH_TEXT_ONLY;
-	
+
 	if (!GEDIT_SEARCH_IS_CASE_SENSITIVE (flags))
 	{
 		search_flags = search_flags | GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
@@ -2351,33 +2431,29 @@ gedit_document_find (GeditDocument* doc, const gchar* str, gint flags)
 	if (GEDIT_SEARCH_IS_FROM_CURSOR (flags))
 	{
 		GtkTextIter sel_bound;
-		
-		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), &iter,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "insert"));
-		
-		gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc), &sel_bound,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "selection_bound"));
-		
+
+		gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),			
+						      &iter,
+						      &sel_bound);
+
 		if (!GEDIT_SEARCH_IS_BACKWARDS (flags))
 			gtk_text_iter_order (&sel_bound, &iter);		
 		else
 			gtk_text_iter_order (&iter, &sel_bound);		
 	}
-	else		
+	else
 	{
 		if (GEDIT_SEARCH_IS_BACKWARDS (flags))
 		{
 			/* get an iterator at the end of the document */
-			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc),
-							    &iter, -1);
+			gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (doc),
+						      &iter);
 		}
 		else
 		{
 			/* get an iterator at the beginning of the document */
-			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc),
-							    &iter, 0);
+			gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (doc),
+							&iter);
 		}
 	}
 	
@@ -2510,6 +2586,7 @@ gedit_document_find_prev (GeditDocument* doc, gint flags)
 gboolean 
 gedit_document_get_selection (GeditDocument *doc, gint *start, gint *end)
 {
+	gboolean ret;
 	GtkTextIter iter;
 	GtkTextIter sel_bound;
 
@@ -2517,15 +2594,10 @@ gedit_document_get_selection (GeditDocument *doc, gint *start, gint *end)
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
 
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &iter,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "insert"));
-		
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &sel_bound,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "selection_bound"));
+	ret = gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),			
+						    &iter,
+						    &sel_bound);
+
 	gtk_text_iter_order (&iter, &sel_bound);	
 
 	if (start != NULL)
@@ -2534,7 +2606,7 @@ gedit_document_get_selection (GeditDocument *doc, gint *start, gint *end)
 	if (end != NULL)
 		*end = gtk_text_iter_get_offset (&sel_bound); 
 
-	return !gtk_text_iter_equal (&sel_bound, &iter);	
+	return ret;
 }
 
 void
@@ -2547,48 +2619,33 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
-	g_return_if_fail (replace != NULL);	
+	g_return_if_fail (replace != NULL);
 
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &iter,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "insert"));
-		
-	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &sel_bound,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "selection_bound"));
-
-	if (gtk_text_iter_equal (&sel_bound, &iter))
+	if (!gtk_text_buffer_get_selection_bounds (GTK_TEXT_BUFFER (doc),			
+						   &iter,
+						   &sel_bound))
 	{
 		gedit_debug (DEBUG_DOCUMENT, "There is no selected text");
 
 		return;
 	}
-	
-	gtk_text_iter_order (&sel_bound, &iter);		
 
 	converted_replace = gedit_utils_convert_search_text (replace);
 
 	gedit_document_begin_user_action (doc);
 
-	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc),
-				&iter,
-				&sel_bound);
+	gtk_text_buffer_delete_selection (GTK_TEXT_BUFFER (doc), FALSE, TRUE);
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &iter,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "insert"));
+			&iter,
+			gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
+
 	if (*converted_replace != '\0')
 		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc),
 					&iter,
 					converted_replace, strlen (converted_replace));
 
-	if (doc->priv->last_replace_text != NULL)
-		g_free (doc->priv->last_replace_text);
-
-	doc->priv->last_replace_text = g_strdup (replace);
+	gedit_document_set_last_replace_text (doc, replace);
 
 	gedit_document_end_user_action (doc);
 
@@ -2651,9 +2708,8 @@ gedit_document_get_cursor (GeditDocument *doc)
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), 0);
 
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
-                                    &iter,
-                                    gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
-					                      "insert"));
+			&iter,
+			gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (doc)));
 
 	return gtk_text_iter_get_offset (&iter); 
 }
@@ -2763,7 +2819,6 @@ gedit_document_set_encoding (GeditDocument *doc, const GeditEncoding *encoding)
 
 	/* FIXME: do we need a encoding changed signal ? - Paolo */
 	g_signal_emit (G_OBJECT (doc), document_signals[NAME_CHANGED], 0);
-
 }
 
 glong
